;/*************************************/;
;/* Example Program 68HC11 & 8255 Port */;
;/*   Bipolar Stepping Motor Control  */;
;/* Hardware : ET-CP68HC11 V1.0       */;
;/* Complier : AS11NEW                */;
;/*************************************/;
;/*=>Motor1 : EN1,EN2,INA,INB,INC,IND */;
;/*  Signal : PA0,PA1,PA2,PA3,PA4,PA5 */;
;/*=>Motor2 : EN1,EN2,INA,INB,INC,IND */;
;/*  Signal : PA6,PA7,PB0,PB1,PB2,PB3 */;
;/*=>Motor3 : EN1,EN2,INA,INB,INC,IND */;
;/*  Signal : PB4,PB5,PB6,PB7,PC0,PC1 */;
;/*=>Motor4 : EN1,EN2,INA,INB,INC,IND */;
;/*  Signal : PC2,PC3,PC4,PC5,PC6,PC7 */;
;/*************************************/;
;
PA          EQU     $1200                   ; Port A of 8255
PB          EQU     $1201                   ; Port B of 8255
PC          EQU     $1202                   ; Port C of 8255
PCC         EQU     $1203                   ; Control Port of 8255

            ORG     $2200
MAIN        LDAA    #$80                    ; Initial 8255 to Output
            STAA    PCC

;/********************/;
;/* Half Step Driver */;
;/********************/;
;
HALF_DRV:   LDAA    #50
            STAA    CYCLECNT
HALF0:      LDAA    #8
            STAA    STEP_CNT                ; 8 Step/1 Cycle Counter
            LDAA    #0
            STAA    STEP_INX
HALF1:      LDX     #TAB_HALF1              ; Half Step Drive Table
            JSR     OUT_STEP                ; Out Signal Drive
            JSR     DELAY                   ; Delay For Next Step
            INC     STEP_INX                ; Next Step
            DEC     STEP_CNT
            BNE     HALF1
            DEC     CYCLECNT
            BNE     HALF0
            ;
            CLRA                            ; off motor
            STAA    PA
            STAA    PB
            STAA    PC
            ;
            PSHX
            PSHB
            LDAB    #10
DLY_OFF1:   LDX     #$0000
DLY_OFF2:   DEX
            BNE     DLY_OFF2
            DECB
            BNE     DLY_OFF1
            PULB
            PULX
            ;
            LDAA    #50
            STAA    CYCLECNT
HALF2:      LDAA    #8
            STAA    STEP_CNT                ; 8 Step/1 Cycle Counter
            LDAA    #0
            STAA    STEP_INX
HALF3:      LDX     #TAB_HALF2              ; Half Step Drive Table
            JSR     OUT_STEP                ; Out Signal Drive
            JSR     DELAY                   ; Delay For Next Step
            INC     STEP_INX                ; Next Step
            DEC     STEP_CNT
            BNE     HALF3
            DEC     CYCLECNT
            BNE     HALF2
            ;
            PSHX
            PSHB
            LDAB    #10
DLY_ON1:    LDX     #$0000
DLY_ON2:    DEX
            BNE     DLY_ON2
            DECB
            BNE     DLY_ON1
            PULB
            PULX
            JMP     HALF_DRV                ; Loop Continue

;/********************************/;
;/* Out Stepp Motor Signal Drive */;
;/* Input  : INX  = Table Signal */;
;/*        : Step Index          */;
;/* Output : Out Port8255 Signal */;
;/********************************/;
;
OUT_STEP:   LDAA    #$0                     ; Clear old Data
            STAA    PA_BUFF
            STAA    PB_BUFF
            STAA    PC_BUFF
            LDAB    STEP_INX
            ABX
            LDAA    0,X                     ; Get Signal
            STAA    PA_BUFF                 ; X,X,D,C,B,A,EN2,EN1 (Motor1)
            TAB                             ;/* Save ACC */
            RORA
            RORA
            RORA
            ANDA    #%11000000              ; EN2,EN1,X,X,X,X,X,X (Motor2)
            ORAA    PA_BUFF                 ; Mix Motor2,Motor1
            STAA    PA_BUFF                 ; EN2,EN1,D,C,B,A,EN2,EN1
            TBA
            RORA
            RORA
            ANDA    #$0F
            STAA    PB_BUFF                 ; X,X,X,X,D,C,B,A (Motor2)
            TBA
            RORA                            ; Swap A
            RORA
            RORA
            RORA
            RORA
            ANDA    #$F0                    ; B,A,EN2,EN1,X,X,X,X (Motor3)
            ORAA    PB_BUFF                 ; Mix Mortor3,Motor2
            STAA    PB_BUFF                 ; B,A,EN2,EN1,D,C,B,A
            TBA
            RORA
            RORA
            RORA
            RORA
            ANDA    #%00000011              ; X,X,X,X,X,X,D,C (Motor3)
            STAA    PC_BUFF
            TBA
            ROLA
            ROLA
            ANDA    #%11111100              ; D,C,B,A,EN2,EN1,X,X (Motor4)
            ORAA    PC_BUFF                 ; Mix Motor4 Motor3
            STAA    PC_BUFF                 ; D,C,B,A,EN2,EN1,D,C
            ;
            LDAA    PA_BUFF                 ; Out Signal
            STAA    PA
            LDAA    PB_BUFF
            STAA    PB
            LDAA    PC_BUFF
            STAA    PC
            RTS

;/**********************/;
;/* Wave-Drive         */;
;/* Half Drive         */;
;/* ONE-Phase-ON Drive */;
;/* ONE-Excitation     */;
;/* 00,D,C,B,A,EN2,EN1 */;
;/* D7..............D0 */;
;/**********************/;
;
TAB_WAVE1:  FCB     00100010B               ; Step-2
            FCB     00000101B               ; Step-4
            FCB     00010010B               ; Step-6
            FCB     00001001B               ; Step-8
            FCB     00100010B               ; Step-2
            FCB     00000101B               ; Step-4
            FCB     00010010B               ; Step-6
            FCB     00001001B               ; Step-8
            ;
TAB_WAVE2:  FCB     00001001B               ; Step-8
            FCB     00010010B               ; Step-6
            FCB     00000101B               ; Step-4
            FCB     00100010B               ; Step-2
            FCB     00001001B               ; Step-8
            FCB     00010010B               ; Step-6
            FCB     00000101B               ; Step-4
            FCB     00100010B               ; Step-2

;/**********************/;
;/* Half Step Drive    */;
;/* ONE-TWO-Excitation */;
;/* 00,D,C,B,A,EN2,EN1 */;
;/* D7..............D0 */;
;/**********************/;
;
TAB_HALF1:  FCB     00101011B               ; Step-1
            FCB     00100010B               ; Step-2
            FCB     00100111B               ; Step-3
            FCB     00000101B               ; Step-4
            FCB     00010111B               ; Step-5
            FCB     00010010B               ; Step-6
            FCB     00011011B               ; Step-7
            FCB     00001001B               ; Step-8
            ;
TAB_HALF2:  FCB     00001001B               ; Step-8
            FCB     00011011B               ; Step-7
            FCB     00010010B               ; Step-6
            FCB     00010111B               ; Step-5
            FCB     00000101B               ; Step-4
            FCB     00100111B               ; Step-3
            FCB     00100010B               ; Step-2
            FCB     00101011B               ; Step-1
            
;/**********************/;
;/* TWO-Phase-ON Drive */;
;/* Normal-Drive       */;
;/* Full Step          */;
;/* TWO-Excitation     */;
;/* 00,D,C,B,A,EN2,EN1 */;
;/* D7..............D0 */;
;/**********************/;
;
TAB_FULL1:  FCB     00101011B               ; Step-1
            FCB     00100111B               ; Step-3
            FCB     00010111B               ; Step-5
            FCB     00011011B               ; Step-7
            FCB     00101011B               ; Step-1
            FCB     00100111B               ; Step-3
            FCB     00010111B               ; Step-5
            FCB     00011011B               ; Step-7
            ;
TAB_FULL2:  FCB     00011011B               ; Step-7
            FCB     00010111B               ; Step-5
            FCB     00100111B               ; Step-3
            FCB     00101011B               ; Step-1
            FCB     00011011B               ; Step-7
            FCB     00010111B               ; Step-5
            FCB     00100111B               ; Step-3
            FCB     00101011B               ; Step-1

;/*******************/;
;/* DELAY TIME SUB. */;
;/*******************/;
;
DELAY:      PSHX
            LDX     #$2000
DLY1:       DEX
            BNE     DLY1
            PULX
            RTS

PA_BUFF:    RMB     1
PB_BUFF:    RMB     1
PC_BUFF:    RMB     1
STEP_CNT:   RMB     1
CYCLECNT:   RMB     1
STEP_INX:   RMB     1

            END
