;/*************************************/;
;/* Example Program MCS51 & 8255 Port */;
;/*   Bipolar Stepping Motor Control  */;
;/* Hardware : ET-CPSB31 V2.0         */;
;/* Complier : SXA51                  */;
;/*************************************/;
;/*=>Motor1 : EN1,EN2,INA,INB,INC,IND */;
;/*  Signal : PA0,PA1,PA2,PA3,PA4,PA5 */;
;/*=>Motor2 : EN1,EN2,INA,INB,INC,IND */;
;/*  Signal : PA6,PA7,PB0,PB1,PB2,PB3 */;
;/*=>Motor3 : EN1,EN2,INA,INB,INC,IND */;
;/*  Signal : PB4,PB5,PB6,PB7,PC0,PC1 */;
;/*=>Motor4 : EN1,EN2,INA,INB,INC,IND */;
;/*  Signal : PC2,PC3,PC4,PC5,PC6,PC7 */;
;/*************************************/;
;
PORTA       EQU     0E0E0H
PORTB       EQU     0E0E1H
PORTC       EQU     0E0E2H
CONTRL      EQU     0E0E3H

            ORG     22H
PA_BUFF:    DS      1
PB_BUFF:    DS      1
PC_BUFF:    DS      1
STEP_CNT:   DS      1
CYCLECNT:   DS      1
STEP_INX:   DS      1
STACK:      DS      32

            ORG     2200H
MAIN:       MOV     SP,#STACK               ; Stack 32 Byte
            MOV     R2,#02H                 ; Power-on Delay
PWR_DLY1:   MOV     R3,#00H
PWR_DLY2:   MOV     R4,#00H
            DJNZ    R4,$
            DJNZ    R3,PWR_DLY2
            DJNZ    R2,PWR_DLY1
            ;
            MOV    DPTR,#CONTRL             ; Control Port 8255
            MOV    A,#80H                   ; PA,PB,PC = Output
            MOVX   @DPTR,A
            ;
            LJMP   HALF_DRV


;/********************/;
;/* Wave Drive Motor */;
;/********************/;
;
WAVE_DRV:   MOV     CYCLECNT,#25
WAVE0:      MOV     STEP_CNT,#8             ; 8 Step/1 Cycle Counter
            MOV     STEP_INX,#0
WAVE1:      MOV     DPTR,#TAB_WAVE1         ; Wave Drive Table
            MOV     A,STEP_INX
            LCALL   OUT_STEP                ; Out Signal Drive
            LCALL   DELAY                   ; Delay For Next Step
            INC     STEP_INX                ; Next Step
            DJNZ    STEP_CNT,WAVE1
            DJNZ    CYCLECNT,WAVE0
            ;
            CLR     A                       ; off motor
            MOV     DPTR,#PORTA             ; Out Signal
            MOVX    @DPTR,A
            MOV     DPTR,#PORTB
            MOVX    @DPTR,A
            MOV     DPTR,#PORTC
            MOVX    @DPTR,A
            ;
            MOV     R3,#20                  ; Delay For Next Cycle
WAVE_DL1:   MOV     R4,#00H
WAVE_DL2:   MOV     R5,#00H
            DJNZ    R5,$
            DJNZ    R4,WAVE_DL2
            DJNZ    R3,WAVE_DL1
            ;
            MOV     CYCLECNT,#25
WAVE2:      MOV     STEP_CNT,#8             ; 8 Step/1 Cycle Counter
            MOV     STEP_INX,#0
WAVE3:      MOV     DPTR,#TAB_WAVE2         ; Wave Drive Table
            MOV     A,STEP_INX
            LCALL   OUT_STEP                ; Out Signal Drive
            LCALL   DELAY                   ; Delay For Next Step
            INC     STEP_INX                ; Next Step
            DJNZ    STEP_CNT,WAVE3
            DJNZ    CYCLECNT,WAVE2
            ;
            CLR     A                       ; off motor
            MOV     DPTR,#PORTA             ; Out Signal
            MOVX    @DPTR,A
            MOV     DPTR,#PORTB
            MOVX    @DPTR,A
            MOV     DPTR,#PORTC
            MOVX    @DPTR,A
            ;
            MOV     R3,#20                  ; Delay For Next Cycle
WAVE_DL3:   MOV     R4,#00H
WAVE_DL4:   MOV     R5,#00H
            DJNZ    R5,$
            DJNZ    R4,WAVE_DL4
            DJNZ    R3,WAVE_DL3
            LJMP    WAVE_DRV                ; Loop Continue

;/********************/;
;/* Full Step Driver */;
;/********************/;
;
FULL_DRV:   MOV     CYCLECNT,#25
FULL0:      MOV     STEP_CNT,#8             ; 8 Step/1 Cycle Counter
            MOV     STEP_INX,#0
FULL1:      MOV     DPTR,#TAB_FULL1         ; Full Step Drive Table
            MOV     A,STEP_INX
            LCALL   OUT_STEP                ; Out Signal Drive
            LCALL   DELAY                   ; Delay For Next Step
            INC     STEP_INX                ; Next Step
            DJNZ    STEP_CNT,FULL1
            DJNZ    CYCLECNT,FULL0
            ;
            CLR     A                       ; off motor
            MOV     DPTR,#PORTA             ; Out Signal
            MOVX    @DPTR,A
            MOV     DPTR,#PORTB
            MOVX    @DPTR,A
            MOV     DPTR,#PORTC
            MOVX    @DPTR,A
            ;
            MOV     R3,#20                  ; Delay For Next Cycle
FULL_DL1:   MOV     R4,#00H
FULL_DL2:   MOV     R5,#00H
            DJNZ    R5,$
            DJNZ    R4,FULL_DL2
            DJNZ    R3,FULL_DL1
            ;
            MOV     CYCLECNT,#25
FULL2:      MOV     STEP_CNT,#8             ; 8 Step/1 Cycle Counter
            MOV     STEP_INX,#0
FULL3:      MOV     DPTR,#TAB_FULL2         ; Full Step Drive Table
            MOV     A,STEP_INX
            LCALL   OUT_STEP                ; Out Signal Drive
            LCALL   DELAY                   ; Delay For Next Step
            INC     STEP_INX                ; Next Step
            DJNZ    STEP_CNT,FULL3
            DJNZ    CYCLECNT,FULL2
            ;
            CLR     A                       ; off motor
            MOV     DPTR,#PORTA             ; Out Signal
            MOVX    @DPTR,A
            MOV     DPTR,#PORTB
            MOVX    @DPTR,A
            MOV     DPTR,#PORTC
            MOVX    @DPTR,A
            ;
            MOV     R3,#20                  ; Delay For Next Cycle
FULL_DL3:   MOV     R4,#00H
FULL_DL4:   MOV     R5,#00H
            DJNZ    R5,$
            DJNZ    R4,FULL_DL4
            DJNZ    R3,FULL_DL3
            LJMP    FULL_DRV                ; Loop Continue

;/********************/;
;/* Half Step Driver */;
;/********************/;
;
HALF_DRV:   MOV     CYCLECNT,#50
HALF0:      MOV     STEP_CNT,#8             ; 8 Step/1 Cycle Counter
            MOV     STEP_INX,#0
HALF1:      MOV     DPTR,#TAB_HALF1         ; Half Step Drive Table
            MOV     A,STEP_INX
            LCALL   OUT_STEP                ; Out Signal Drive
            LCALL   DELAY                   ; Delay For Next Step
            INC     STEP_INX                ; Next Step
            DJNZ    STEP_CNT,HALF1
            DJNZ    CYCLECNT,HALF0
            ;
            CLR     A                       ; off motor
            MOV     DPTR,#PORTA             ; Out Signal
            MOVX    @DPTR,A
            MOV     DPTR,#PORTB
            MOVX    @DPTR,A
            MOV     DPTR,#PORTC
            MOVX    @DPTR,A
            ;
            MOV     R3,#20                  ; Delay For Next Cycle
HALF_DL1:   MOV     R4,#00H
HALF_DL2:   MOV     R5,#00H
            DJNZ    R5,$
            DJNZ    R4,HALF_DL2
            DJNZ    R3,HALF_DL1
            ;
            MOV     CYCLECNT,#50
HALF2:      MOV     STEP_CNT,#8             ; 8 Step/1 Cycle Counter
            MOV     STEP_INX,#0
HALF3:      MOV     DPTR,#TAB_HALF2         ; Half Step Drive Table
            MOV     A,STEP_INX
            LCALL   OUT_STEP                ; Out Signal Drive
            LCALL   DELAY                   ; Delay For Next Step
            INC     STEP_INX                ; Next Step
            DJNZ    STEP_CNT,HALF3
            DJNZ    CYCLECNT,HALF2
            ;
            CLR     A                       ; off motor
            MOV     DPTR,#PORTA             ; Out Signal
            MOVX    @DPTR,A
            MOV     DPTR,#PORTB
            MOVX    @DPTR,A
            MOV     DPTR,#PORTC
            MOVX    @DPTR,A
            ;
            MOV     R3,#20                  ; Delay For Next Cycle
HALF_DL3:   MOV     R4,#00H
HALF_DL4:   MOV     R5,#00H
            DJNZ    R5,$
            DJNZ    R4,HALF_DL4
            DJNZ    R3,HALF_DL3
            LJMP    HALF_DRV                ; Loop Continue

;/********************************/;
;/* Out Stepp Motor Signal Drive */;
;/* Input  : DPTR = Table Signal */;
;/*        : ACC  = Step Index   */;
;/* Output : Out Port8255 Signal */;
;/********************************/;
;
OUT_STEP:   MOV     PA_BUFF,#0              ; Clear old Data
            MOV     PB_BUFF,#0
            MOV     PC_BUFF,#0
            MOVC    A,@A+DPTR               ; Get Signal
            MOV     PA_BUFF,A               ; X,X,D,C,B,A,EN2,EN1 (Motor1)
            MOV     B,A                     ;/* Save ACC */
            RR      A
            RR      A
            ANL     A,#11000000B            ; EN2,EN1,X,X,X,X,X,X (Motor2)
            ORL     A,PA_BUFF               ; Mix Motor2,Motor1
            MOV     PA_BUFF,A               ; EN2,EN1,D,C,B,A,EN2,EN1
            MOV     A,B
            RR      A
            RR      A
            ANL     A,#0FH
            MOV     PB_BUFF,A               ; X,X,X,X,D,C,B,A (Motor2)
            MOV     A,B
            SWAP    A
            ANL     A,#0F0H                 ; B,A,EN2,EN1,X,X,X,X (Motor3)
            ORL     A,PB_BUFF               ; Mix Mortor3,Motor2
            MOV     PB_BUFF,A               ; B,A,EN2,EN1,D,C,B,A
            MOV     A,B
            RR      A
            RR      A
            RR      A
            RR      A
            ANL     A,#00000011B            ; X,X,X,X,X,X,D,C (Motor3)
            MOV     PC_BUFF,A
            MOV     A,B
            RL      A
            RL      A
            ANL     A,#11111100B            ; D,C,B,A,EN2,EN1,X,X (Motor4)
            ORL     A,PC_BUFF               ; Mix Motor4 Motor3
            MOV     PC_BUFF,A               ; D,C,B,A,EN2,EN1,D,C
            MOV     DPTR,#PORTA             ; Out Signal
            MOV     A,PA_BUFF
            MOVX    @DPTR,A
            MOV     DPTR,#PORTB
            MOV     A,PB_BUFF
            MOVX    @DPTR,A
            MOV     DPTR,#PORTC
            MOV     A,PC_BUFF
            MOVX    @DPTR,A
            RET

;/**********************/;
;/* Wave-Drive         */;
;/* Half Drive         */;
;/* ONE-Phase-ON Drive */;
;/* ONE-Excitation     */;
;/* 00,D,C,B,A,EN2,EN1 */;
;/* D7..............D0 */;
;/**********************/;
;
TAB_WAVE1:  DB      00100010B               ; Step-2
            DB      00000101B               ; Step-4
            DB      00010010B               ; Step-6
            DB      00001001B               ; Step-8
            DB      00100010B               ; Step-2
            DB      00000101B               ; Step-4
            DB      00010010B               ; Step-6
            DB      00001001B               ; Step-8
            ;
TAB_WAVE2:  DB      00001001B               ; Step-8
            DB      00010010B               ; Step-6
            DB      00000101B               ; Step-4
            DB      00100010B               ; Step-2
            DB      00001001B               ; Step-8
            DB      00010010B               ; Step-6
            DB      00000101B               ; Step-4
            DB      00100010B               ; Step-2

;/**********************/;
;/* Half Step Drive    */;
;/* ONE-TWO-Excitation */;
;/* 00,D,C,B,A,EN2,EN1 */;
;/* D7..............D0 */;
;/**********************/;
;
TAB_HALF1:  DB      00101011B               ; Step-1
            DB      00100010B               ; Step-2
            DB      00100111B               ; Step-3
            DB      00000101B               ; Step-4
            DB      00010111B               ; Step-5
            DB      00010010B               ; Step-6
            DB      00011011B               ; Step-7
            DB      00001001B               ; Step-8
            ;
TAB_HALF2:  DB      00001001B               ; Step-8
            DB      00011011B               ; Step-7
            DB      00010010B               ; Step-6
            DB      00010111B               ; Step-5
            DB      00000101B               ; Step-4
            DB      00100111B               ; Step-3
            DB      00100010B               ; Step-2
            DB      00101011B               ; Step-1
            
;/**********************/;
;/* TWO-Phase-ON Drive */;
;/* Normal-Drive       */;
;/* Full Step          */;
;/* TWO-Excitation     */;
;/* 00,D,C,B,A,EN2,EN1 */;
;/* D7..............D0 */;
;/**********************/;
;
TAB_FULL1:  DB      00101011B               ; Step-1
            DB      00100111B               ; Step-3
            DB      00010111B               ; Step-5
            DB      00011011B               ; Step-7
            DB      00101011B               ; Step-1
            DB      00100111B               ; Step-3
            DB      00010111B               ; Step-5
            DB      00011011B               ; Step-7
            ;
TAB_FULL2:  DB      00011011B               ; Step-7
            DB      00010111B               ; Step-5
            DB      00100111B               ; Step-3
            DB      00101011B               ; Step-1
            DB      00011011B               ; Step-7
            DB      00010111B               ; Step-5
            DB      00100111B               ; Step-3
            DB      00101011B               ; Step-1

;/**************/;
;/* Delay Time */;
;/**************/;
;
DELAY:      MOV     R4,#50
DLY1:       MOV     R5,#00H
            DJNZ    R5,$
            DJNZ    R4,DLY1
            RET

            END

